% Program demo_6_3
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
clear all, close all
% Sampling rate alteration by the fractional factor R = 16/15
 
% Generating the input signal x[n]
n = -2:32;
f1 = 0.1; f2 = 0.16;
x = n+2*sin(pi*f1*n)+5*cos(pi*f2*n)+1; 
n=0:length(x)-5;
figure (1)
subplot(2,1,1),stem(n,x(3:length(x)-2),'k');
xlabel('Time index [n]'), ylabel('x[n]')
 
% Defining fractional intervals for sampling rate alteration by 16/15
R = 16/15;                                 %  Re-sampling factor
s = 1:15;
mu = [0,1-(1-1/R)*s];
mu = repmat(mu,1,3);               % Vector of fractional intervals
 
% Farrow filter, coefficients: 
C0 = [1/6,-1/2,1/2,-1/6];            % Vector of Lagrange coefficients for C_0(k)
C1 = [0,1/2,-1,1/2];                   % Vector of Lagrange coefficients for C_1(k)   
C2 = ([-1/6,1,-1/2,-1/3]);            % Vector of Lagrange coefficients for C_2(k)
C3 = [0,0,1];                              % Vector of Lagrange coefficients for C_3(k)
 
% Farrow filter, initial states: 
xs0 = [0,0,0]; xs1 = [0,0,0]; xs2 = [0,0,0]; xs3 = [0,0]; 
 
% Farrow filtering:
l=1;
for n=1:1:31
    xnl = x(n+3);
[v0,xs0] = filter(C0,1,xnl,xs0);
[v1,xs1] = filter(C1,1,xnl,xs1);
[v2,xs2] = filter(C2,1,xnl,xs2);
[v3,xs3] = filter(C3,1,xnl,xs3);
if mu(l)==0
     y(l) = ((v0*mu(l)+v1)*mu(l)+v2)*mu(l)+x(n+2);
    y(l+1) = ((v0*mu(l+1)+v1)*mu(l+1)+v2)*mu(l+1)+x(n+2);
    l = l + 2;
else
    y(l) = ((v0*mu(l)+v1)*mu(l)+v2)*mu(l)+x(n+2);
    l = l + 1;
end
end
figure (1)
subplot(2,1,2)
stem(0:length(y) -1,y,'sk'), xlabel ('Time index [l]'),ylabel('y[l]')
axis([0,33,0,40])
